@@ -97,17 +97,20 @@ nbsite_gallery_conf = {
             'title': 'Component Gallery',
             'sections': [
                 'panes',
+                'widgets',
                 'layouts',
-                'templates',
+                # 3 most important by expected usage. Rest alphabetically
+                'chat',
                 'global',
                 'indicators',
-                'widgets',
+                'templates',
             ],
             'titles': {
                 'Vega': 'Altair & Vega',
                 'DeckGL': 'PyDeck & Deck.gl',
                 'ECharts': 'PyEcharts & ECharts',
-                'IPyWidget': 'ipywidgets'
+                'IPyWidget': 'ipywidgets',
+                'PanelCallbackHandler': 'LangChain CallbackHandler',
             },
             'as_pyodide': True,
             'normalize_titles': False
