@@ -21,6 +21,9 @@ from bokeh.core.has_props import _default_resolver
 from bokeh.document import Document
 from bokeh.model import Model
 from bokeh.settings import settings as bk_settings
+from param.display import (
+    register_display_accessor, unregister_display_accessor,
+)
 from pyviz_comms import (
     JupyterCommManager as _JupyterCommManager, extension as _pyviz_extension,
 )
@@ -298,6 +301,9 @@ class _config(_base_config):
     _oauth_extra_params = param.Dict(default={}, doc="""
         Additional parameters required for OAuth provider.""")
 
+    _oauth_refresh_tokens = param.Boolean(default=False, doc="""
+        Whether to automatically refresh access tokens in the background.""")
+
     _inline = param.Boolean(default=_LOCAL_DEV_VERSION, allow_None=True, doc="""
         Whether to inline JS and CSS resources. If disabled, resources
         are loaded from CDN if one is available.""")
@@ -311,7 +317,7 @@ class _config(_base_config):
         'nthreads', 'oauth_provider', 'oauth_expiry', 'oauth_key',
         'oauth_secret', 'oauth_jwt_user', 'oauth_redirect_uri',
         'oauth_encryption_key', 'oauth_extra_params', 'npm_cdn',
-        'layout_compatibility'
+        'layout_compatibility', 'oauth_refresh_tokens'
     ]
 
     _truthy = ['True', 'true', '1', True, 1]
@@ -556,6 +562,13 @@ class _config(_base_config):
     def oauth_jwt_user(self):
         return os.environ.get('PANEL_OAUTH_JWT_USER', self._oauth_jwt_user)
 
+    @property
+    def oauth_refresh_tokens(self):
+        refresh = os.environ.get('PANEL_OAUTH_REFRESH_TOKENS', self._oauth_refresh_tokens)
+        if isinstance(refresh, bool):
+            return refresh
+        return refresh.lower() in ('1', 'true')
+
     @property
     def oauth_encryption_key(self):
         return os.environ.get('PANEL_OAUTH_ENCRYPTION', self._oauth_encryption_key)
@@ -773,7 +786,17 @@ class panel_extension(_pyviz_extension):
         except Exception:
             return
 
-        from .io.notebook import load_notebook
+        from .io.notebook import load_notebook, mime_renderer
+
+        try:
+            unregister_display_accessor('_ipython_display_')
+        except KeyError:
+            pass
+
+        try:
+            register_display_accessor('_repr_mimebundle_', mime_renderer)
+        except Exception:
+            pass
 
         self._detect_comms(params)
 
@@ -790,7 +813,7 @@ class panel_extension(_pyviz_extension):
             # In embedded mode the ipywidgets_bokeh model must be loaded
             __import__(self._imports['ipywidgets'])
 
-        nb_loaded = getattr(self, '_repeat_execution_in_cell', False)
+        nb_loaded = published = getattr(self, '_repeat_execution_in_cell', False)
         if 'holoviews' in sys.modules:
             if getattr(hv.extension, '_loaded', False):
                 nb_loaded = True
@@ -808,13 +831,18 @@ class panel_extension(_pyviz_extension):
                 config.inline, reloading=nb_loaded
             )
 
-        if not nb_loaded and config.browser_info and state.browser_info:
+        if not published:
+            self._display_globals()
+
+    @staticmethod
+    def _display_globals():
+        if config.browser_info and state.browser_info:
             doc = Document()
             comm = state._comm_manager.get_server_comm()
             model = state.browser_info._render_model(doc, comm)
             bundle, meta = state.browser_info._render_mimebundle(model, doc, comm)
             display(bundle, metadata=meta, raw=True)  # noqa
-        if not nb_loaded and config.notifications:
+        if config.notifications:
             display(state.notifications)  # noqa
 
     def _detect_comms(self, params):
