@@ -74,6 +74,7 @@ from ..config import config
 from ..util import edit_readonly, fullpath
 from ..util.warnings import warn
 from .document import init_doc, unlocked, with_lock  # noqa
+from .liveness import LivenessHandler
 from .loading import LOADING_INDICATOR_CSS_CLASS
 from .logging import (
     LOG_SESSION_CREATED, LOG_SESSION_DESTROYED, LOG_SESSION_LAUNCHING,
@@ -207,7 +208,7 @@ def _initialize_session_info(session_context: 'BokehSessionContext'):
     }
     state.param.trigger('session_info')
 
-state.on_session_created(_initialize_session_info)
+state._on_session_created_internal.append(_initialize_session_info)
 
 #---------------------------------------------------------------------
 # Bokeh patches
@@ -401,7 +402,7 @@ class Application(BkApplication):
         with set_curdoc(session_context._document):
             if self._admin is not None:
                 config._admin = self._admin
-            for cb in state._on_session_created:
+            for cb in state._on_session_created_internal+state._on_session_created:
                 cb(session_context)
         await super().on_session_created(session_context)
 
@@ -480,6 +481,35 @@ class DocHandler(LoginUrlMixin, BkDocHandler, SessionPrefixHandler):
                     session.block_expiration()
         return session
 
+    def _token_payload(self):
+        app = self.application
+        if app.include_headers is None:
+            excluded_headers = (app.exclude_headers or [])
+            allowed_headers = [header for header in self.request.headers
+                               if header not in excluded_headers]
+        else:
+            allowed_headers = app.include_headers
+        headers = {k: v for k, v in self.request.headers.items()
+                   if k in allowed_headers}
+
+        if app.include_cookies is None:
+            excluded_cookies = (app.exclude_cookies or [])
+            allowed_cookies = [cookie for cookie in self.request.cookies
+                               if cookie not in excluded_cookies]
+        else:
+            allowed_cookies = app.include_cookies
+        cookies = {k: v.value for k, v in self.request.cookies.items()
+                   if k in allowed_cookies}
+
+        if cookies and 'Cookie' in headers and 'Cookie' not in (app.include_headers or []):
+            # Do not include Cookie header since cookies can be restored from cookies dict
+            del headers['Cookie']
+
+        arguments = {} if self.request.arguments is None else self.request.arguments
+        payload = {'headers': headers, 'cookies': cookies, 'arguments': arguments}
+        payload.update(self.application_context.application.process_request(self.request))
+        return payload
+
     @authenticated
     async def get(self, *args, **kwargs):
         app = self.application
@@ -493,6 +523,7 @@ class DocHandler(LoginUrlMixin, BkDocHandler, SessionPrefixHandler):
                     signed=self.application.sign_sessions
                 )
                 payload = get_token_payload(session.token)
+                payload.update(self._token_payload())
                 del payload['session_expiry']
                 token = generate_jwt_token(
                     session_id,
@@ -524,7 +555,10 @@ class DocHandler(LoginUrlMixin, BkDocHandler, SessionPrefixHandler):
                     auth_error = f'{state.user} is not authorized to access this application.'
                     try:
                         authorized = auth_cb(*auth_args)
-                        if not authorized:
+                        if isinstance(authorized, str):
+                            self.redirect(authorized)
+                            return
+                        elif not authorized:
                             auth_error = (
                                 f'Authorization callback errored. Could not validate user name "{state.user}" '
                                 f'for the given app "{self.request.path}".'
@@ -985,8 +1019,14 @@ def get_server(
     oauth_error_template: Optional[str] = None,
     cookie_secret: Optional[str] = None,
     oauth_encryption_key: Optional[str] = None,
+    oauth_jwt_user: Optional[str] = None,
+    oauth_refresh_tokens: Optional[bool] = None,
+    login_endpoint: Optional[str] = None,
+    logout_endpoint: Optional[str] = None,
+    login_template: Optional[str] = None,
     logout_template: Optional[str] = None,
     session_history: Optional[int] = None,
+    liveness: bool | str = False,
     **kwargs
 ) -> Server:
     """
@@ -1038,15 +1078,26 @@ def get_server(
       The client secret for the OAuth provider
     oauth_redirect_uri: Optional[str] = None,
       Overrides the default OAuth redirect URI
+    oauth_jwt_user: Optional[str] = None,
+      Key that identifies the user in the JWT id_token.
     oauth_extra_params: dict (optional, default={})
       Additional information for the OAuth provider
     oauth_error_template: str (optional, default=None)
       Jinja2 template used when displaying authentication errors.
     cookie_secret: str (optional, default=None)
       A random secret string to sign cookies (required for OAuth)
-    oauth_encryption_key: str (optional, default=False)
+    oauth_encryption_key: str (optional, default=None)
       A random encryption key used for encrypting OAuth user
       information and access tokens.
+    oauth_refresh_tokens: bool (optional, default=None)
+      Whether to automatically refresh OAuth access tokens when they expire.
+    login_endpoint: str (optional, default=None)
+      Overrides the default login endpoint `/login`
+    logout_endpoint: str (optional, default=None)
+      Overrides the default logout endpoint `/logout`
+    logout_template: str (optional, default=None)
+      Jinja2 template served when viewing the login endpoint when
+      authentication is enabled.
     logout_template: str (optional, default=None)
       Jinja2 template served when viewing the logout endpoint when
       authentication is enabled.
@@ -1055,6 +1106,10 @@ def get_server(
       and non-None value will launch a REST endpoint at
       /rest/session_info, which returns information about the session
       history.
+    liveness: bool | str (optional, default=False)
+      Whether to add a liveness endpoint. If a string is provided
+      then this will be used as the endpoint, otherwise the endpoint
+      will be hosted at /liveness.
     kwargs: dict
       Additional keyword arguments to pass to Server instance.
 
@@ -1071,6 +1126,8 @@ def get_server(
     if isinstance(panel, dict):
         apps = {}
         for slug, app in panel.items():
+            if slug.endswith('/') and not slug == '/':
+                raise ValueError(f"Invalid URL: trailing slash '/' used for {slug!r} not supported.")
             if isinstance(title, dict):
                 try:
                     title_ = title[slug]
@@ -1133,6 +1190,10 @@ def get_server(
         extra_patterns.extend(pattern)
         state.publish('session_info', state, ['session_info'])
 
+    if liveness:
+        liveness_endpoint = 'liveness' if isinstance(liveness, bool) else liveness
+        extra_patterns += [(r"/%s" % liveness_endpoint, LivenessHandler, dict(applications=apps))]
+
     opts = dict(kwargs)
     if loop:
         asyncio.set_event_loop(loop.asyncio_loop)
@@ -1154,20 +1215,21 @@ def get_server(
     # Configure OAuth
     from ..config import config
     server_config = {}
-    if basic_auth:
-        from ..auth import BasicProvider
-        server_config['basic_auth'] = basic_auth
-        basic_login_template = kwargs.pop('basic_login_template', None)
-        opts['auth_provider'] = BasicProvider(
-            basic_login_template,
-            logout_template=logout_template
-        )
-    elif oauth_provider:
-        from ..auth import OAuthProvider
-        config.oauth_provider = oauth_provider # type: ignore
-        opts['auth_provider'] = OAuthProvider(
-            error_template=oauth_error_template,
-            logout_template=logout_template
+    login_template = kwargs.pop('basic_login_template', login_template)
+    if basic_auth or oauth_provider:
+        from ..auth import BasicAuthProvider, OAuthProvider
+        if basic_auth:
+            server_config['basic_auth'] = basic_auth
+            provider = BasicAuthProvider
+        else:
+            config.oauth_provider = oauth_provider
+            provider = OAuthProvider
+        opts['auth_provider'] = provider(
+            login_endpoint=login_endpoint,
+            logout_endpoint=logout_endpoint,
+            login_template=login_template,
+            logout_template=logout_template,
+            error_template=oauth_error_template
         )
     if oauth_key:
         config.oauth_key = oauth_key # type: ignore
@@ -1179,6 +1241,10 @@ def get_server(
         config.cookie_secret = cookie_secret # type: ignore
     if oauth_redirect_uri:
         config.oauth_redirect_uri = oauth_redirect_uri # type: ignore
+    if oauth_refresh_tokens is not None:
+        config.oauth_refresh_tokens = oauth_refresh_tokens
+    if oauth_jwt_user is not None:
+        config.oauth_jwt_user = oauth_jwt_user
     opts['cookie_secret'] = config.cookie_secret
 
     server = Server(apps, port=port, **opts)
@@ -1191,8 +1257,9 @@ def get_server(
     state._server_config[server._tornado] = server_config
 
     if show:
+        login_endpoint = login_endpoint or '/login'
         def show_callback():
-            server.show('/login' if config.oauth_provider else '/')
+            server.show(login_endpoint if config.oauth_provider or basic_auth else '/')
         server.io_loop.add_callback(show_callback)
 
     def sig_exit(*args, **kwargs):
