@@ -7,21 +7,21 @@ import asyncio
 import datetime as dt
 import functools
 import inspect
-import json
 import logging
 import shutil
 import sys
 import threading
 import time
 
-from collections import OrderedDict, defaultdict
+from collections import Counter, OrderedDict, defaultdict
 from collections.abc import Iterator
 from contextlib import contextmanager
 from contextvars import ContextVar
 from functools import partial, wraps
 from typing import (
     TYPE_CHECKING, Any, Callable, ClassVar, Coroutine, Dict,
-    Iterator as TIterator, List, Optional, Tuple, Type, TypeVar, Union,
+    Iterator as TIterator, List, Literal, Optional, Tuple, Type, TypeVar,
+    Union,
 )
 from urllib.parse import urljoin
 from weakref import WeakKeyDictionary
@@ -33,7 +33,7 @@ from bokeh.document.locking import UnlockedDocumentProxy
 from bokeh.io import curdoc as _curdoc
 from pyviz_comms import CommManager as _CommManager
 
-from ..util import base64url_decode, parse_timedelta
+from ..util import decode_token, parse_timedelta
 from .logging import LOG_SESSION_RENDERED, LOG_USER_MSG
 
 _state_logger = logging.getLogger('panel.state')
@@ -48,7 +48,6 @@ if TYPE_CHECKING:
     from IPython.display import DisplayHandle
     from pyviz_comms import Comm
     from tornado.ioloop import IOLoop
-    from typing_extensions import Literal
 
     from ..template.base import BaseTemplate
     from ..viewable import Viewable
@@ -177,6 +176,8 @@ class _state(param.Parameterized):
     # Dictionary of callbacks to be triggered on app load
     _onload: ClassVar[Dict[Document, Callable[[], None]]] = WeakKeyDictionary()
     _on_session_created: ClassVar[List[Callable[[BokehSessionContext], []]]] = []
+    _on_session_created_internal: ClassVar[List[Callable[[BokehSessionContext], []]]] = []
+    _on_session_destroyed: ClassVar[List[Callable[[BokehSessionContext], []]]] = []
     _loaded: ClassVar[WeakKeyDictionary[Document, bool]] = WeakKeyDictionary()
 
     # Module that was run during setup
@@ -209,6 +210,10 @@ class _state(param.Parameterized):
     _sessions = {}
     _session_key_funcs = {}
 
+    # Override user info
+    _oauth_user_overrides = {}
+    _active_users = Counter()
+
     def __repr__(self) -> str:
         server_info = []
         for server, panel, docs in self._servers.values():
@@ -437,6 +442,10 @@ class _state(param.Parameterized):
         else:
             self.log(f'Exception of unknown type raised: {exception}', level='error')
 
+    def _register_session_destroyed(self, session_context: BokehSessionContext):
+        for cb in self._on_session_destroyed:
+            session_context._document.on_session_destroyed(cb)
+
     #----------------------------------------------------------------
     # Public Methods
     #----------------------------------------------------------------
@@ -677,6 +686,13 @@ class _state(param.Parameterized):
         """
         Callback that is triggered when a session is created.
         """
+        if self.curdoc and self.curdoc.session_context:
+            raise RuntimeError(
+                "Cannot register session creation callback from within a session. "
+                "If running a Panel application from the CLI set up the callback "
+                "in a --setup script, if starting a server dynamically set it up "
+                "before starting the server."
+            )
         self._on_session_created.append(callback)
 
     def on_session_destroyed(self, callback: Callable[[BokehSessionContext], None]) -> None:
@@ -687,10 +703,9 @@ class _state(param.Parameterized):
         if doc:
             doc.on_session_destroyed(callback)
         else:
-            raise RuntimeError(
-                "Could not add session destroyed callback since no "
-                "document to attach it to could be found."
-            )
+            if self._register_session_destroyed not in self._on_session_created:
+                self._on_session_created.append(self._register_session_destroyed)
+            self._on_session_destroyed.append(callback)
 
     def publish(
         self, endpoint: str, parameterized: param.Parameterized,
@@ -742,6 +757,8 @@ class _state(param.Parameterized):
             self._thread_pool = None
         self._sessions.clear()
         self._session_key_funcs.clear()
+        self._on_session_created.clear()
+        self._on_session_destroyed.clear()
 
     def schedule_task(
         self, name: str, callback: Callable[[], None], at: Tat =None,
@@ -867,7 +884,7 @@ class _state(param.Parameterized):
     # Public Properties
     #----------------------------------------------------------------
 
-    def _decode_cookie(self, cookie_name):
+    def _decode_cookie(self, cookie_name, cookie=None):
         from tornado.web import decode_signed_value
 
         from ..config import config
@@ -875,6 +892,9 @@ class _state(param.Parameterized):
         if cookie is None:
             return None
         cookie = decode_signed_value(config.cookie_secret, cookie_name, cookie)
+        return self._decrypt_cookie(cookie)
+
+    def _decrypt_cookie(self, cookie):
         if self.encryption is None:
             return cookie.decode('utf-8')
         return self.encryption.decrypt(cookie).decode('utf-8')
@@ -884,7 +904,15 @@ class _state(param.Parameterized):
         """
         Returns the OAuth access_token if enabled.
         """
-        return self._decode_cookie('access_token')
+        if self.user in self._oauth_user_overrides and 'refresh_token' in self._oauth_user_overrides:
+            return self._oauth_user_overrides[self.user]['access_token']
+        access_token = self._decode_cookie('access_token')
+        if not access_token:
+            return
+        decoded_token = decode_token(access_token)
+        if decoded_token['exp'] <= dt.datetime.now(dt.timezone.utc).timestamp():
+            return None
+        return access_token
 
     @property
     def app_url(self) -> str | None:
@@ -964,25 +992,15 @@ class _state(param.Parameterized):
     def location(self) -> Location | None:
         if self.curdoc and self.curdoc not in self._locations:
             from .location import Location
-            loc = self._locations[self.curdoc] = Location()
+            if self.curdoc.session_context:
+                loc = Location.from_request(self.curdoc.session_context.request)
+            else:
+                loc = Location()
+            self._locations[self.curdoc] = loc
         elif self.curdoc is None:
             loc = self._location
         else:
             loc = self._locations.get(self.curdoc) if self.curdoc else None
-        if loc is None:
-            return loc
-
-        if '?' in self.base_url:
-            try:
-                loc.search = f'?{self.base_url.split("?")[-1].strip("/")}'
-            except Exception:
-                pass
-        if '#' in self.base_url:
-            try:
-                loc.hash = f'#{self.base_url.split("#")[-1].strip("/")}'
-            except Exception:
-                pass
-
         return loc
 
     @property
@@ -1012,7 +1030,18 @@ class _state(param.Parameterized):
         """
         Returns the OAuth refresh_token if enabled and available.
         """
-        return self._decode_cookie('refresh_token')
+        if self.user in self._oauth_user_overrides and 'refresh_token' in self._oauth_user_overrides:
+            return self._oauth_user_overrides[self.user]['refresh_token']
+        refresh_token = self._decode_cookie('refresh_token')
+        if not refresh_token:
+            return
+        try:
+            decoded_token = decode_token(refresh_token)
+        except ValueError:
+            return refresh_token
+        if decoded_token['exp'] <= dt.datetime.now(dt.timezone.utc).timestamp():
+            return None
+        return refresh_token
 
     @property
     def served(self):
@@ -1064,23 +1093,7 @@ class _state(param.Parameterized):
         """
         Returns the OAuth user information if enabled.
         """
-        from tornado.web import decode_signed_value
-
-        from ..config import config
-        id_token = self.cookies.get('id_token')
-        if id_token is None or config.cookie_secret is None:
-            return None
-        id_token = decode_signed_value(config.cookie_secret, 'id_token', id_token)
-        if self.encryption is None:
-            id_token = id_token
-        else:
-            id_token = self.encryption.decrypt(id_token)
-        if b"." in id_token:
-            signing_input, _ = id_token.rsplit(b".", 1)
-            _, payload_segment = signing_input.split(b".", 1)
-        else:
-            payload_segment = id_token
-        return json.loads(base64url_decode(payload_segment).decode('utf-8'))
-
+        id_token = self._decode_cookie('id_token')
+        return decode_token(id_token)
 
 state = _state()
