@@ -209,7 +209,10 @@ class Tabs(NamedListPanel):
             else:
                 try:
                     rendered[pref] = child = pane._get_model(doc, root, model, comm)
-                except RerenderError:
+                except RerenderError as e:
+                    if e.layout is not None and e.layout is not self:
+                        raise e
+                    e.layout = None
                     return self._get_objects(model, current_objects[:i], doc, root, comm)
 
             panel = panels[pref] = BkTabPanel(
