@@ -6,7 +6,8 @@ import {ColorMapper} from "@bokehjs/models/mappers/color_mapper"
 import {Enum} from "@bokehjs/core/kinds"
 
 import {HTMLBox, HTMLBoxView, set_size} from "../layout"
-import {vtkns, setup_vtkns, VolumeType, majorAxis, applyStyle, CSSProperties, Annotation} from "./util"
+import type {VolumeType, CSSProperties, Annotation} from "./util"
+import {vtkns, setup_vtkns, majorAxis, applyStyle} from "./util"
 import {VTKColorBar} from "./vtkcolorbar"
 import {VTKAxes} from "./vtkaxes"
 
