@@ -42,11 +42,12 @@ def port_open(port):
     sock.close()
     return is_open
 
+
 def get_default_port():
-    # to get a different starting port per worker for pytest-xdist
+    worker_count = int(os.environ.get("PYTEST_XDIST_WORKER_COUNT", "1"))
     worker_id = os.environ.get("PYTEST_XDIST_WORKER", "0")
-    n = int(re.sub(r"\D", "", worker_id))
-    return 6000 + n * 30
+    worker_idx = int(re.sub(r"\D", "", worker_id))
+    return 9001 + (worker_idx * worker_count * 10)
 
 def start_jupyter():
     global JUPYTER_PORT, JUPYTER_PROCESS
@@ -163,8 +164,11 @@ def comm():
 
 @pytest.fixture
 def port():
-    PORT[0] += 1
-    return PORT[0]
+    worker_id = os.environ.get("PYTEST_XDIST_WORKER", "0")
+    worker_count = int(os.environ.get("PYTEST_XDIST_WORKER_COUNT", "1"))
+    new_port = PORT[0] + int(re.sub(r"\D", "", worker_id))
+    PORT[0] += worker_count
+    return new_port
 
 
 @pytest.fixture
