@@ -24,7 +24,7 @@ from panel.models.tabulator import TableEditEvent
 from panel.pane import Markdown
 from panel.reactive import ReactiveHTML
 from panel.template import BootstrapTemplate
-from panel.tests.util import wait_until
+from panel.tests.util import serve_and_request, serve_and_wait, wait_until
 from panel.widgets import (
     Button, Tabulator, Terminal, TextInput,
 )
@@ -60,116 +60,77 @@ def test_server_change_io_state(html_server_session):
     html._server_change(session.document, None, None, 'text', '<h1>Title</h1>', '<h1>New Title</h1>')
 
 
-def test_server_static_dirs(port):
+def test_server_static_dirs():
     html = Markdown('# Title')
 
     static = {'tests': os.path.dirname(__file__)}
-    serve(html, port=port, threaded=True, static_dirs=static, show=False)
 
-    # Wait for server to start
-    time.sleep(1)
+    r = serve_and_request(html, static_dirs=static, suffix="/tests/test_server.py")
 
-    r = requests.get(f"http://localhost:{port}/tests/test_server.py")
     with open(__file__, encoding='utf-8') as f:
         assert f.read() == r.content.decode('utf-8').replace('\r\n', '\n')
 
 
-def test_server_root_handler(port):
+def test_server_root_handler():
     html = Markdown('# Title')
 
-    serve(
-        {'app': html}, port=port, threaded=True, show=False, use_index=True,
-        index=INDEX_HTML, redirect_root=False
+    r = serve_and_request(
+        {'app': html}, use_index=True, index=INDEX_HTML, redirect_root=False
     )
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/")
-
     assert 'href="./app"' in r.content.decode('utf-8')
 
 
-def test_server_template_static_resources(port):
+def test_server_template_static_resources():
     template = BootstrapTemplate()
 
-    serve({'template': template}, port=port, threaded=True, show=False)
+    r = serve_and_request({'template': template}, suffix="/static/extensions/panel/bundled/bootstraptemplate/bootstrap.css")
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/static/extensions/panel/bundled/bootstraptemplate/bootstrap.css")
     with open(DIST_DIR / 'bundled' / 'bootstraptemplate' / 'bootstrap.css', encoding='utf-8') as f:
         assert f.read() == r.content.decode('utf-8').replace('\r\n', '\n')
 
 
-def test_server_template_static_resources_with_prefix(port):
+def test_server_template_static_resources_with_prefix():
     template = BootstrapTemplate()
 
-    serve({'template': template}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'template': template}, prefix="/prefix", suffix="/prefix/static/extensions/panel/bundled/bootstraptemplate/bootstrap.css")
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/static/extensions/panel/bundled/bootstraptemplate/bootstrap.css")
     with open(DIST_DIR / 'bundled' / 'bootstraptemplate' / 'bootstrap.css', encoding='utf-8') as f:
         assert f.read() == r.content.decode('utf-8').replace('\r\n', '\n')
 
 
-def test_server_template_static_resources_with_prefix_relative_url(port):
+def test_server_template_static_resources_with_prefix_relative_url():
     template = BootstrapTemplate()
 
-    serve({'template': template}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'template': template}, prefix='/prefix', suffix="/prefix/template")
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/template")
-    content = r.content.decode('utf-8')
-    assert 'href="static/extensions/panel/bundled/bootstraptemplate/bootstrap.css"' in content
+    assert 'href="static/extensions/panel/bundled/bootstraptemplate/bootstrap.css"' in r.content.decode('utf-8')
 
 
-def test_server_template_static_resources_with_subpath_and_prefix_relative_url(port):
+def test_server_template_static_resources_with_subpath_and_prefix_relative_url():
     template = BootstrapTemplate()
 
-    serve({'/subpath/template': template}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'/subpath/template': template}, prefix='/prefix', suffix="/prefix/subpath/template")
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/subpath/template")
-    content = r.content.decode('utf-8')
-    assert 'href="../static/extensions/panel/bundled/bootstraptemplate/bootstrap.css"' in content
+    assert 'href="../static/extensions/panel/bundled/bootstraptemplate/bootstrap.css"' in r.content.decode('utf-8')
 
 
-def test_server_extensions_on_root(port):
-    html = Markdown('# Title')
-
-    serve(html, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/static/extensions/panel/css/loading.css")
-    assert r.ok
+def test_server_extensions_on_root():
+    md = Markdown('# Title')
+    assert serve_and_request(md).ok
 
 
 def test_autoload_js(port):
     html = Markdown('# Title')
     app_name = 'test'
-    serve({app_name: html}, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(0.5)
-
     args = f"bokeh-autoload-element=1002&bokeh-app-path=/{app_name}&bokeh-absolute-url=http://localhost:{port}/{app_name}"
-    r = requests.get(f"http://localhost:{port}/{app_name}/autoload.js?{args}")
+    r = serve_and_request({app_name: html}, port=port, suffix=f"/{app_name}/autoload.js?{args}")
 
     assert r.status_code == 200
     assert f"http://localhost:{port}/static/extensions/panel/panel.min.js" in r.content.decode('utf-8')
 
 
-def test_server_async_callbacks(port):
+def test_server_async_callbacks():
     button = Button(name='Click')
 
     counts = []
@@ -182,26 +143,18 @@ def test_server_async_callbacks(port):
 
     button.on_click(cb)
 
-    serve(button, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
+    serve_and_request(button)
 
     doc = list(button._models.values())[0][0].document
     with set_curdoc(doc):
         for _ in range(5):
             button.clicks += 1
 
-    # Wait for callbacks to be scheduled
-    time.sleep(2)
-
-    # Ensure multiple callbacks started concurrently
-    assert max(counts) > 1
+    # Checks whether Button on_click callback was executed concurrently
+    wait_until(lambda: len(counts) > 0 and max(counts) > 1)
 
 
-def test_server_async_local_state(port):
+def test_server_async_local_state():
     docs = {}
 
     async def task():
@@ -216,23 +169,14 @@ def test_server_async_local_state(port):
         state.execute(task)
         return 'My app'
 
-    serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    for i in range(3):
-        requests.get(f"http://localhost:{port}/")
-
-    # Wait for callbacks to be scheduled
-    time.sleep(2)
+    serve_and_request(app, n=3)
 
     # Ensure state.curdoc was consistent despite asyncio context switching
-    assert len(docs) == 3
-    assert all([len(set(docs)) == 1 and docs[0] is doc for doc, docs in docs.items()])
+    wait_until(lambda: len(docs) == 3)
+    wait_until(lambda: all([len(set(docs)) == 1 and docs[0] is doc for doc, docs in docs.items()]))
 
 
-def test_server_async_local_state_nested_tasks(port):
+def test_server_async_local_state_nested_tasks():
     docs = {}
 
     async def task(depth=1):
@@ -249,20 +193,11 @@ def test_server_async_local_state_nested_tasks(port):
         state.execute(task)
         return 'My app'
 
-    serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    for i in range(3):
-        requests.get(f"http://localhost:{port}/")
-
-    # Wait for callbacks to be scheduled
-    time.sleep(2)
+    serve_and_request(app, n=3)
 
     # Ensure state.curdoc was consistent despite asyncio context switching
-    assert len(docs) == 3
-    assert all(len(set(docs)) == 1 and docs[0] is doc for doc, docs in docs.items())
+    wait_until(lambda: len(docs) == 3)
+    wait_until(lambda: all(len(set(docs)) == 1 and docs[0] is doc for doc, docs in docs.items()))
 
 
 def test_serve_config_per_session_state():
@@ -273,13 +208,9 @@ def test_serve_config_per_session_state():
     def app2():
         config.raw_css = [CSS2]
 
-
     port1, port2 = 7001, 7002
-    serve(app1, port=port1, threaded=True, show=False)
-    serve(app2, port=port2, threaded=True, show=False)
-
-    # Wait for servers to start
-    time.sleep(1)
+    serve_and_wait(app1, port=port1)
+    serve_and_wait(app2, port=port2)
 
     r1 = requests.get(f"http://localhost:{port1}/").content.decode('utf-8')
     r2 = requests.get(f"http://localhost:{port2}/").content.decode('utf-8')
@@ -292,19 +223,40 @@ def test_serve_config_per_session_state():
     assert CSS2 in r2
 
 
+def test_server_on_session_created():
+    session_contexts = []
+    def append_session(session_context):
+        session_contexts.append(session_context)
+    state.on_session_created(append_session)
+
+    html = Markdown('# Title')
+
+    serve_and_request(html, n=3)
+
+    assert len(session_contexts) == 3
+
+
+def test_server_on_session_destroyed():
+    session_contexts = []
+    def append_session(session_context):
+        session_contexts.append(session_context)
+    state.on_session_destroyed(append_session)
+
+    html = Markdown('# Title')
+
+    serve_and_request(html, n=3, check_unused_sessions_milliseconds=500, unused_session_lifetime_milliseconds=500)
+
+    wait_until(lambda: len(session_contexts) == 3)
+
+
 # This test seem to fail if run after:
 # - test_server_async_local_state_nested_tasks
 # - test_server_async_local_state
-def test_server_session_info(port):
+def test_server_session_info():
     with config.set(session_history=-1):
         html = Markdown('# Title')
 
-        serve(html, port=port, threaded=True, show=False)
-
-        # Wait for server to start
-        time.sleep(1)
-
-        requests.get(f"http://localhost:{port}/")
+        serve_and_request(html)
 
         assert state.session_info['total'] == 1
         assert len(state.session_info['sessions']) == 1
@@ -329,7 +281,7 @@ def test_server_session_info(port):
     assert state.session_info['live'] == 0
 
 
-def test_server_schedule_repeat(port):
+def test_server_schedule_repeat():
     state.cache['count'] = 0
     def periodic_cb():
         state.cache['count'] += 1
@@ -338,22 +290,12 @@ def test_server_schedule_repeat(port):
         state.schedule_task('periodic', periodic_cb, period='0.5s')
         return '# state.schedule test'
 
-    server = serve(app, port=port, threaded=True, show=False)
+    serve_and_request(app)
 
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-
-    # Wait for periodic execution
-    time.sleep(1)
-
-    assert state.cache['count']
-
-    server.stop()
+    wait_until(lambda: state.cache['count'] > 0)
 
 
-def test_server_schedule_at(port):
+def test_server_schedule_at():
     def periodic_cb():
         state.cache['at'] = dt.datetime.now()
 
@@ -363,25 +305,15 @@ def test_server_schedule_at(port):
         state.schedule_task('periodic', periodic_cb, at=scheduled)
         return '# state.schedule test'
 
-    server = serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-
-    # Wait for callback to be executed
-    time.sleep(1)
+    serve_and_request(app)
 
     # Check callback was executed within small margin of error
-    assert 'at' in state.cache
+    wait_until(lambda: 'at' in state.cache)
     assert abs(state.cache['at'] - scheduled) < dt.timedelta(seconds=0.2)
     assert len(state._scheduled) == 0
 
-    server.stop()
 
-
-def test_server_schedule_at_iterator(port):
+def test_server_schedule_at_iterator():
     state.cache['at'] = []
     def periodic_cb():
         state.cache['at'].append(dt.datetime.now())
@@ -397,26 +329,16 @@ def test_server_schedule_at_iterator(port):
         state.schedule_task('periodic', periodic_cb, at=schedule())
         return '# state.schedule test'
 
-    server = serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-
-    # Wait for callbacks to be executed
-    time.sleep(1)
+    serve_and_request(app)
 
     # Check callbacks were executed within small margin of error
-    assert len(state.cache['at']) == 2
+    wait_until(lambda: len(state.cache['at']) == 2)
     assert abs(state.cache['at'][0] - scheduled1) < dt.timedelta(seconds=0.2)
     assert abs(state.cache['at'][1] - scheduled2) < dt.timedelta(seconds=0.2)
     assert len(state._scheduled) == 0
 
-    server.stop()
 
-
-def test_server_schedule_at_callable(port):
+def test_server_schedule_at_callable():
     state.cache['at'] = []
     def periodic_cb():
         state.cache['at'].append(dt.datetime.now())
@@ -434,44 +356,29 @@ def test_server_schedule_at_callable(port):
         state.schedule_task('periodic', periodic_cb, at=schedule)
         return '# state.schedule test'
 
-    server = serve(app, port=port, threaded=True, show=False)
+    serve_and_request(app)
 
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-
-    # Wait for callback to be executed
-    time.sleep(1)
+    # Check callbacks were executed within small margin of error
+    wait_until(lambda: len(state.cache['at']) == 2)
 
     # Convert scheduled times to local time
     scheduled = [
         s.replace(tzinfo=dt.timezone.utc).astimezone().replace(tzinfo=None)
         for s in scheduled
     ]
-
-    # Check callbacks were executed within small margin of error
-    assert len(state.cache['at']) == 2
     assert abs(state.cache['at'][0] - scheduled[0]) < dt.timedelta(seconds=0.2)
     assert abs(state.cache['at'][1] - scheduled[1]) < dt.timedelta(seconds=0.2)
     assert len(state._scheduled) == 0
 
-    server.stop()
 
 @pytest.mark.xdist_group(name="server")
-def test_server_reuse_sessions(port, reuse_sessions):
+def test_server_reuse_sessions(reuse_sessions):
     def app(counts=[0]):
         content = f'# Count {counts[0]}'
         counts[0] += 1
         return content
 
-    serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    r1 = requests.get(f"http://localhost:{port}/")
-    r2 = requests.get(f"http://localhost:{port}/")
+    r1, r2 = serve_and_request(app, n=2)
 
     assert len(state._sessions) == 1
     assert ('/', 'default') in state._sessions
@@ -496,10 +403,7 @@ def test_server_reuse_sessions_with_session_key_func(port, reuse_sessions):
         counts[0] += 1
         return tmpl
 
-    serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
+    serve_and_wait(app, port=port)
 
     r1 = requests.get(f"http://localhost:{port}/?arg=foo")
     r2 = requests.get(f"http://localhost:{port}/?arg=bar")
@@ -521,6 +425,7 @@ def test_show_server_info(html_server_session, markdown_server_session):
     assert f"localhost:{html_port} - HTML" in server_info
     assert f"localhost:{markdown_port} - Markdown" in server_info
 
+
 @pytest.mark.xdist_group(name="server")
 def test_kill_all_servers(html_server_session, markdown_server_session):
     _, server_1, *_ = html_server_session
@@ -529,6 +434,7 @@ def test_kill_all_servers(html_server_session, markdown_server_session):
     assert server_1._stopped
     assert server_2._stopped
 
+
 @pytest.mark.xdist_group(name="server")
 def test_multiple_titles(multiple_apps_server_sessions):
     """Serve multiple apps with a title per app."""
@@ -555,7 +461,6 @@ def test_serve_can_serve_bokeh_app_from_file():
     assert "/bk-app" in server._tornado.applications
 
 
-@pytest.mark.flaky(max_runs=3)
 def test_server_thread_pool_change_event(threads, port):
     button = Button(name='Click')
     button2 = Button(name='Click')
@@ -572,12 +477,7 @@ def test_server_thread_pool_change_event(threads, port):
     button2.on_click(cb)
     layout = Row(button, button2)
 
-    serve(layout, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
+    serve_and_request(layout)
 
     model = list(layout._models.values())[0][0]
     doc = model.document
@@ -586,8 +486,7 @@ def test_server_thread_pool_change_event(threads, port):
         button2._server_change(doc, model.ref['id'], None, 'clicks', 0, 1)
 
     # Checks whether Button on_click callback was executed concurrently
-    wait_until(lambda: len(counts) > 0)
-    wait_until(lambda: max(counts) == 2)
+    wait_until(lambda: len(counts) > 0 and max(counts) > 1)
 
 
 def test_server_thread_pool_bokeh_event(threads, port):
@@ -607,23 +506,15 @@ def test_server_thread_pool_bokeh_event(threads, port):
 
     tabulator.on_edit(cb)
 
-    serve(tabulator, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
+    serve_and_request(tabulator)
 
     model = list(tabulator._models.values())[0][0]
     event = TableEditEvent(model, 'A', 0)
     for _ in range(5):
         tabulator._server_event(model.document, event)
 
-    # Wait for callbacks to be scheduled
-    time.sleep(1)
-
     # Checks whether Tabulator on_edit callback was executed concurrently
-    assert max(counts) > 1
+    wait_until(lambda: len(counts) > 0 and max(counts) > 1)
 
 
 def test_server_thread_pool_periodic(threads, port):
@@ -640,22 +531,10 @@ def test_server_thread_pool_periodic(threads, port):
         state.add_periodic_callback(cb, 100)
         return button
 
-    serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-
-    # Wait for callbacks to be scheduled
-    time.sleep(1)
+    serve_and_request(app)
 
     # Checks whether periodic callbacks were executed concurrently
-    def _more_than_two():
-        if counts:
-            assert max(counts) >= 2
-
-    wait_until(_more_than_two)
+    wait_until(lambda: len(counts) > 0 and max(counts) > 1)
 
 
 def test_server_thread_pool_onload(threads, port):
@@ -678,18 +557,10 @@ def test_server_thread_pool_onload(threads, port):
 
         return button
 
-    serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-    requests.get(f"http://localhost:{port}/")
-
-    time.sleep(1)
+    serve_and_request(app, n=2)
 
     # Checks whether onload callbacks were executed concurrently
-    assert max(counts) >= 2
+    wait_until(lambda: len(counts) > 0 and max(counts) > 1)
 
 
 def test_server_thread_pool_busy(threads, port):
@@ -709,17 +580,9 @@ def test_server_thread_pool_busy(threads, port):
         state.curdoc.add_next_tick_callback(simulate_click)
         return button
 
-    serve(app, port=port, threaded=True, show=False)
+    serve_and_request(app, suffix="/")
 
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-
-    time.sleep(1)
-
-    assert state._busy_counter == 0
-    assert state.busy == False
+    wait_until(lambda: state._busy_counter == 0 and not state.busy)
 
 
 def test_server_async_onload(threads, port):
@@ -742,18 +605,10 @@ def test_server_async_onload(threads, port):
 
         return button
 
-    serve(app, port=port, threaded=True, show=False)
-
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
-    requests.get(f"http://localhost:{port}/")
-
-    time.sleep(1)
+    serve_and_request(app, n=2)
 
     # Checks whether onload callbacks were executed concurrently
-    assert max(counts) >= 2
+    wait_until(lambda: len(counts) and max(counts) >= 2)
 
 
 class CustomBootstrapTemplate(BootstrapTemplate):
@@ -764,12 +619,8 @@ class CustomBootstrapTemplate(BootstrapTemplate):
 def test_server_template_custom_resources(port):
     template = CustomBootstrapTemplate()
 
-    serve({'template': template}, port=port, threaded=True, show=False)
+    r = serve_and_request({'template': template}, suffix="/components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css")
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css")
     with open(pathlib.Path(__file__).parent / 'assets' / 'custom.css', encoding='utf-8') as f:
         assert f.read() == r.content.decode('utf-8').replace('\r\n', '\n')
 
@@ -777,12 +628,8 @@ def test_server_template_custom_resources(port):
 def test_server_template_custom_resources_with_prefix(port):
     template = CustomBootstrapTemplate()
 
-    serve({'template': template}, port=port, threaded=True, show=False, prefix='prefix')
-
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css")
+    path = "/prefix/components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css"
+    r = serve_and_request({'template': template}, prefix='/prefix', suffix=path)
     with open(pathlib.Path(__file__).parent / 'assets' / 'custom.css', encoding='utf-8') as f:
         assert f.read() == r.content.decode('utf-8').replace('\r\n', '\n')
 
@@ -790,27 +637,17 @@ def test_server_template_custom_resources_with_prefix(port):
 def test_server_template_custom_resources_with_prefix_relative_url(port):
     template = CustomBootstrapTemplate()
 
-    serve({'template': template}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'template': template}, prefix='/prefix', suffix='/prefix/template')
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/template")
-    content = r.content.decode('utf-8')
-    assert 'href="components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css"' in content
+    assert 'href="components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css"' in r.content.decode('utf-8')
 
 
 def test_server_template_custom_resources_with_subpath_and_prefix_relative_url(port):
     template = CustomBootstrapTemplate()
 
-    serve({'/subpath/template': template}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'/subpath/template': template}, prefix='/prefix', suffix='/prefix/subpath/template')
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/subpath/template")
-    content = r.content.decode('utf-8')
-    assert 'href="../components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css"' in content
+    assert 'href="../components/panel.tests.test_server/CustomBootstrapTemplate/_css/assets/custom.css"' in r.content.decode('utf-8')
 
 
 class CustomComponent(ReactiveHTML):
@@ -823,12 +660,9 @@ class CustomComponent(ReactiveHTML):
 def test_server_component_custom_resources(port):
     component = CustomComponent()
 
-    serve({'component': component}, port=port, threaded=True, show=False)
+    path = "/components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css"
+    r = serve_and_request({'component': component}, suffix=path)
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css")
     with open(pathlib.Path(__file__).parent / 'assets' / 'custom.css', encoding='utf-8') as f:
         assert f.read() == r.content.decode('utf-8').replace('\r\n', '\n')
 
@@ -836,12 +670,10 @@ def test_server_component_custom_resources(port):
 def test_server_component_custom_resources_with_prefix(port):
     component = CustomComponent()
 
-    serve({'component': component}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request(
+        {'component': component}, prefix='/prefix', suffix="/prefix/components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css"
+    )
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css")
     with open(pathlib.Path(__file__).parent / 'assets' / 'custom.css', encoding='utf-8') as f:
         assert f.read() == r.content.decode('utf-8').replace('\r\n', '\n')
 
@@ -849,53 +681,33 @@ def test_server_component_custom_resources_with_prefix(port):
 def test_server_component_custom_resources_with_prefix_relative_url(port):
     component = CustomComponent()
 
-    serve({'component': component}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'component': component}, prefix='/prefix', suffix='/prefix/component')
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/component")
-    content = r.content.decode('utf-8')
-    assert 'href="components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css"' in content
+    assert 'href="components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css"' in r.content.decode('utf-8')
 
 
 def test_server_component_custom_resources_with_subpath_and_prefix_relative_url(port):
     component = CustomComponent()
 
-    serve({'/subpath/component': component}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'/subpath/component': component}, prefix='/prefix', suffix='/prefix/subpath/component')
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/subpath/component")
-    content = r.content.decode('utf-8')
-    assert 'href="../components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css"' in content
+    assert 'href="../components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css"' in r.content.decode('utf-8')
 
 
 def test_server_component_css_with_prefix_relative_url(port):
     component = Terminal()
 
-    serve({'component': component}, port=port, threaded=True, show=False)
+    r = serve_and_request({'component': component}, suffix='/component')
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/component")
-    content = r.content.decode('utf-8')
-    assert 'href="static/extensions/panel/bundled/terminal/xterm@4.11.0/css/xterm.css' in content
+    assert 'href="static/extensions/panel/bundled/terminal/xterm@4.11.0/css/xterm.css' in r.content.decode('utf-8')
 
 
 def test_server_component_css_with_subpath_and_prefix_relative_url(port):
     component = Terminal()
 
-    serve({'/subpath/component': component}, port=port, threaded=True, show=False, prefix='prefix')
+    r = serve_and_request({'/subpath/component': component}, prefix='/prefix', suffix='/prefix/subpath/component')
 
-    # Wait for server to start
-    time.sleep(1)
-
-    r = requests.get(f"http://localhost:{port}/prefix/subpath/component")
-    content = r.content.decode('utf-8')
-    assert 'href="../static/extensions/panel/bundled/terminal/xterm@4.11.0/css/xterm.css' in content
+    assert 'href="../static/extensions/panel/bundled/terminal/xterm@4.11.0/css/xterm.css' in r.content.decode('utf-8')
 
 
 def synchronous_handler(event=None):
@@ -930,15 +742,10 @@ def test_server_exception_handler_bokeh_event(threads, handler, port, request):
         state.curdoc.add_next_tick_callback(simulate_click)
         return button
 
-    serve(app, port=port, threaded=True, show=False)
+    serve_and_request(app)
 
-    time.sleep(1)
+    wait_until(lambda: len(exceptions) == 1)
 
-    requests.get(f"http://localhost:{port}")
-
-    time.sleep(0.5)
-
-    assert len(exceptions) == 1
 
 @pytest.mark.parametrize(
     'threads, handler', [
@@ -966,15 +773,9 @@ def test_server_exception_handler_async_change_event(threads, handler, port, req
         state.curdoc.add_next_tick_callback(simulate_input)
         return text_input
 
-    serve(app, port=port, threaded=True, show=False)
+    serve_and_request(app)
 
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}")
-
-    time.sleep(0.5)
-
-    assert len(exceptions) == 1
+    wait_until(lambda: len(exceptions) == 1)
 
 
 @pytest.mark.parametrize(
@@ -1003,15 +804,9 @@ def test_server_exception_handler_async_onload_event(threads, handler, port, req
         state.curdoc.add_next_tick_callback(loaded)
         return text_input
 
-    serve(app, port=port, threaded=True, show=False)
+    serve_and_request(app)
 
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}")
-
-    time.sleep(0.5)
-
-    assert len(exceptions) == 1
+    wait_until(lambda: len(exceptions) == 1)
 
 
 def test_server_no_warning_empty_layout(port, caplog):
