@@ -1,5 +1,3 @@
-import time
-
 import pytest
 
 try:
@@ -9,19 +7,15 @@ except ImportError:
     pytestmark = pytest.mark.skip('playwright not available')
 
 from panel.config import config
-from panel.io.server import serve
 from panel.pane import Markdown
+from panel.tests.util import serve_component
 
 
-def test_global_loading_indicator(page, port):
+def test_global_loading_indicator(page):
     def app():
         config.global_loading_spinner = True
         return Markdown('Blah')
 
-    serve(app, port=port, threaded=True, show=False)
-
-    time.sleep(0.5)
-
-    page.goto(f"http://localhost:{port}")
+    serve_component(page, app)
 
     expect(page.locator("body")).not_to_have_class('pn-loading')
