@@ -3,7 +3,7 @@ import pytest
 from bokeh.models import Div
 
 from panel import Accordion
-from panel.tests.util import serve_panel_widget
+from panel.tests.util import serve_component
 
 try:
     from playwright.sync_api import expect
@@ -33,10 +33,10 @@ def is_expanded(card_object, card_content):
     return True
 
 
-def test_accordion_default(page, port, accordion_components):
+def test_accordion_default(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     # there are 2 card in this accordion
@@ -63,13 +63,13 @@ def test_accordion_default(page, port, accordion_components):
     assert is_expanded(card_object=d1_object, card_content=d1.text)
 
 
-def test_accordion_card_name(page, port, accordion_components):
+def test_accordion_card_name(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(
         ('Card 0', d0),
         ('Card 1', d1),
     )
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     d0_object = accordion_elements.nth(0)
@@ -79,10 +79,10 @@ def test_accordion_card_name(page, port, accordion_components):
     expect(d1_object).to_contain_text('Card 1')
 
 
-def test_accordion_active(page, port, accordion_components):
+def test_accordion_active(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1, active=[0])
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     # there are 2 card in this accordion
@@ -99,10 +99,10 @@ def test_accordion_active(page, port, accordion_components):
     assert is_collapsed(card_object=d1_object, card_content=d1.text)
 
 
-def test_accordion_objects(page, port, accordion_components):
+def test_accordion_objects(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     # change the entire list of objects in the accordion
     new_objects = [d0]
@@ -111,10 +111,10 @@ def test_accordion_objects(page, port, accordion_components):
     expect(accordion_elements).to_have_count(len(new_objects))
 
 
-def test_accordion_toggle(page, port, accordion_components):
+def test_accordion_toggle(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1, toggle=True)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     # there are 2 card in this accordion
@@ -137,9 +137,9 @@ def test_accordion_toggle(page, port, accordion_components):
     assert is_expanded(card_object=d1_object, card_content=d1.text)
 
 
-def test_accordion_append(page, port, accordion_components):
+def test_accordion_append(page, accordion_components):
     accordion = Accordion()
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     # empty accordion
@@ -163,10 +163,10 @@ def test_accordion_append(page, port, accordion_components):
     assert is_collapsed(card_object=d1_object, card_content=d1.text)
 
 
-def test_accordion_extend(page, port, accordion_components):
+def test_accordion_extend(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     expect(accordion_elements).to_have_count(len(accordion_components))
@@ -186,10 +186,10 @@ def test_accordion_extend(page, port, accordion_components):
     assert is_collapsed(card_object=d2_object, card_content=d2.text)
 
 
-def test_accordion_clear(page, port, accordion_components):
+def test_accordion_clear(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     expect(accordion_elements).to_have_count(len(accordion_components))
@@ -200,10 +200,10 @@ def test_accordion_clear(page, port, accordion_components):
     expect(accordion_elements).to_have_count(0)
 
 
-def test_accordion_insert(page, port, accordion_components):
+def test_accordion_insert(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     expect(accordion_elements).to_have_count(len(accordion_components))
@@ -230,10 +230,10 @@ def test_accordion_insert(page, port, accordion_components):
     expect(d1_object).to_contain_text(d1.name)
 
 
-def test_accordion_pop(page, port, accordion_components):
+def test_accordion_pop(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     expect(accordion_elements).to_have_count(len(accordion_components))
@@ -246,10 +246,10 @@ def test_accordion_pop(page, port, accordion_components):
     expect(d1_object).to_contain_text(d1.name)
 
 
-def test_accordion_remove(page, port, accordion_components):
+def test_accordion_remove(page, accordion_components):
     d0, d1 = accordion_components
     accordion = Accordion(d0, d1)
-    serve_panel_widget(page, port, accordion)
+    serve_component(page, accordion)
 
     accordion_elements = page.locator('.accordion')
     expect(accordion_elements).to_have_count(len(accordion_components))
