@@ -1,7 +1,7 @@
 import pytest
 
 from panel import Card
-from panel.tests.util import serve_panel_widget
+from panel.tests.util import serve_component
 from panel.widgets import FloatSlider, TextInput
 
 try:
@@ -20,10 +20,10 @@ def card_components():
     return w1, w2
 
 
-def test_card_default(page, port, card_components):
+def test_card_default(page, card_components):
     w1, w2 = card_components
     card = Card(w1, w2)
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     card_elements = page.locator('.card > div, .card > button')
     # the card is expanded as default and includes a header and its inner objects
@@ -42,10 +42,10 @@ def test_card_default(page, port, card_components):
     assert button_name == "\u25bc"
 
 
-def test_card_collapsed(page, port, card_components):
+def test_card_collapsed(page, card_components):
     w1, w2 = card_components
     card = Card(w1, w2)
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     card_elements = page.locator('.card > div, .card > button')
     card_button = page.locator('.card-button')
@@ -64,10 +64,10 @@ def test_card_collapsed(page, port, card_components):
     assert card_button.inner_text() == "\u25bc"
 
 
-def test_card_not_collapsible(page, port, card_components):
+def test_card_not_collapsible(page, card_components):
     w1, w2 = card_components
     card = Card(w1, w2, collapsible=False)
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     # no card button to disable collapsing the card
     card_button = page.locator('.card-button')
@@ -77,10 +77,10 @@ def test_card_not_collapsible(page, port, card_components):
     expect(card_elements).to_have_count(len(card) + 1)
 
 
-def test_card_hide_header(page, port, card_components):
+def test_card_hide_header(page, card_components):
     w1, w2 = card_components
     card = Card(w1, w2, hide_header=True)
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     # no card header
     card_header = page.locator('.card-header')
@@ -90,63 +90,60 @@ def test_card_hide_header(page, port, card_components):
     expect(card_elements).to_have_count(len(card))
 
 
-def test_card_objects(page, port, card_components):
+def test_card_objects(page, card_components):
     w1, w2 = card_components
     card = Card(w1, w2)
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
-    new_objects = [w2]
-    # set new list of objects for the card
-    card.objects = new_objects
+    card.objects = [w2]
 
     card_elements = page.locator('.card > div, .card > button')
     expect(card_elements).to_have_count(2)
 
     card_header = card_elements.nth(0)
     w2_object = card_elements.nth(1)
-    assert 'card-header' in card_header.get_attribute('class')
-    assert 'class_w2' in w2_object.get_attribute('class')
+    expect(card_header).to_have_class('card-header')
+    expect(w2_object).to_have_class('bk-TextInput class_w2')
 
     w3 = TextInput(name='Text:', css_classes=['class_w3'])
     card.append(w3)
-    card_elements = page.locator('.card > div, .card > button')
+
     expect(card_elements).to_have_count(3)
-    w3_object = card_elements.nth(2)
-    assert 'class_w3' in w3_object.get_attribute('class')
+    expect(card_elements.nth(2)).to_have_class('bk-TextInput class_w3')
 
 
-def test_card_title(page, port, card_components):
+def test_card_title(page, card_components):
     w1, w2 = card_components
     card_title = 'Card Title'
     card = Card(w1, w2, title=card_title)
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
-    assert page.locator('.card-title').locator("div").inner_text() == card_title
+    expect(page.locator('.card-title').locator("div")).to_have_text(card_title)
 
 
-def test_card_background(page, port, card_components):
+def test_card_background(page, card_components):
     w1, w2 = card_components
     background = 'rgb(128, 128, 128)'
     card = Card(w1, w2, styles=dict(background=background))
 
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     card_widget = page.locator('.card')
-    assert f'background: {background};' in card_widget.get_attribute('style')
+    expect(card_widget).to_have_css('background-color', background)
 
 
-def test_card_background_legacy(page, port, card_components):
+def test_card_background_legacy(page, card_components):
     w1, w2 = card_components
     background = 'rgb(128, 128, 128)'
     card = Card(w1, w2, background=background)
 
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     card_widget = page.locator('.card')
-    assert f'background: {background};' in card_widget.get_attribute('style')
+    expect(card_widget).to_have_css('background-color', background)
 
 
-def test_card_header_color_formatting(page, port):
+def test_card_header_color_formatting(page):
     header_color = 'rgb(0, 0, 128)'
     active_header_background = 'rgb(0, 128, 0)'
     header_background = 'rgb(128, 0, 0)'
@@ -155,21 +152,21 @@ def test_card_header_color_formatting(page, port):
         active_header_background=active_header_background,
         header_background=header_background,
     )
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     card_header = page.locator('.card-header')
-    assert f'color: {header_color};' in card_header.get_attribute('style')
+    expect(card_header).to_have_css('color', header_color)
 
     # card is expanded by default
-    assert f'background-color: {active_header_background};' in card_header.get_attribute('style')
+    expect(card_header).to_have_css('background-color', active_header_background)
 
     # collapse the card, header background color is set to header_background
     card_header.wait_for()
     card_header.click()
-    assert f'background-color: {header_background};' in card_header.get_attribute('style')
+    expect(card_header).to_have_css('background-color', header_background)
 
 
-def test_card_custom_css(page, port):
+def test_card_custom_css(page):
     # mock css classes to test the Card setting
     additional_css_class = 'css_class'
     additional_header_css_class = 'header_css_class'
@@ -181,7 +178,7 @@ def test_card_custom_css(page, port):
     card.header_css_classes.append(additional_header_css_class)
     card.button_css_classes.append(additional_button_css_class)
 
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
     card_widget = page.locator(f'.card.{additional_css_class}')
     expect(card_widget).to_have_count(1)
@@ -193,9 +190,8 @@ def test_card_custom_css(page, port):
     expect(card_button).to_have_count(1)
 
 
-def test_card_scrollable(page, port):
+def test_card_scrollable(page):
     card = Card(scroll=True)
-    serve_panel_widget(page, port, card)
+    serve_component(page, card)
 
-    card_widget = page.locator('.card')
-    assert 'scrollable-vertical' in card_widget.get_attribute('class')
+    expect(page.locator('.card')).to_have_class('bk-panel-models-layout-Card card scrollable-vertical')
