@@ -1,5 +1,3 @@
-import time
-
 import pytest
 
 try:
@@ -8,25 +6,20 @@ try:
 except ImportError:
     pytestmark = pytest.mark.skip('playwright not available')
 
-from panel.io.server import serve
 from panel.pane import Markdown
 from panel.template import FastGridTemplate
+from panel.tests.util import serve_component
 
 
-def test_fast_grid_template_no_console_errors(page, port):
+def test_fast_grid_template_no_console_errors(page):
     tmpl = FastGridTemplate()
     md = Markdown('Initial')
 
     tmpl.main[0:3, 0:3] = md
 
-    serve(tmpl, port=port, threaded=True, show=False)
+    msgs, _ = serve_component(page, tmpl)
 
-    time.sleep(0.2)
-
-    msgs = []
-    page.on("console", lambda msg: msgs.append(msg))
-
-    page.goto(f"http://localhost:{port}", timeout=40_000)
+    expect(page.locator(".markdown").locator("div")).to_have_text('Initial\n')
 
     expected = ['maxWidth', 'maxHeight']
     assert [
@@ -34,17 +27,13 @@ def test_fast_grid_template_no_console_errors(page, port):
     ] == []
 
 
-def test_fast_grid_template_updates(page, port):
+def test_fast_grid_template_updates(page):
     tmpl = FastGridTemplate()
     md = Markdown('Initial')
 
     tmpl.main[0:3, 0:3] = md
 
-    serve(tmpl, port=port, threaded=True, show=False)
-
-    time.sleep(0.2)
-
-    page.goto(f"http://localhost:{port}", timeout=40_000)
+    serve_component(page, tmpl)
 
     expect(page.locator(".markdown").locator("div")).to_have_text('Initial\n')
     md.object = 'Updated'
