@@ -1,5 +1,3 @@
-import time
-
 import pytest
 
 try:
@@ -8,40 +6,31 @@ try:
 except ImportError:
     pytestmark = pytest.mark.skip('playwright not available')
 
-from panel.io.server import serve
 from panel.pane import Markdown
 from panel.template import SlidesTemplate
+from panel.tests.util import serve_component
 
 
-def test_slides_template_no_console_errors(page, port):
+def test_slides_template_no_console_errors(page):
     tmpl = SlidesTemplate()
     md = Markdown('Initial')
 
     tmpl.main.append(md)
 
-    serve(tmpl, port=port, threaded=True, show=False)
+    msgs, _ = serve_component(page, tmpl)
 
-    time.sleep(0.5)
-
-    msgs = []
-    page.on("console", lambda msg: msgs.append(msg))
-
-    page.goto(f"http://localhost:{port}")
+    expect(page.locator(".markdown").locator("div")).to_have_text('Initial\n')
 
     assert [msg for msg in msgs if msg.type == 'error'] == []
 
 
-def test_slides_template_updates(page, port):
+def test_slides_template_updates(page):
     tmpl = SlidesTemplate()
     md = Markdown('Initial')
 
     tmpl.main.append(md)
 
-    serve(tmpl, port=port, threaded=True, show=False)
-
-    time.sleep(0.2)
-
-    page.goto(f"http://localhost:{port}")
+    serve_component(page, tmpl)
 
     expect(page.locator(".markdown").locator("div")).to_have_text('Initial\n')
     md.object = 'Updated'
