@@ -1,11 +1,9 @@
 import datetime
-import time
 
 import pytest
 
-from panel.io.server import serve
-from panel.tests.util import wait_until
-from panel.widgets import DatetimePicker, DatetimeRangePicker
+from panel.tests.util import serve_component, wait_until
+from panel.widgets import DatetimePicker, DatetimeRangePicker, TextAreaInput
 
 try:
     from playwright.sync_api import expect
@@ -141,11 +139,10 @@ def valid_next_month(old_month, new_month, old_year, new_year):
     return True
 
 
-def test_datetimepicker_default(page, port, weekdays_as_str, months_as_str):
+def test_datetimepicker_default(page, weekdays_as_str, months_as_str):
     datetime_picker_widget = DatetimePicker()
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     datetime_picker = page.locator('.flatpickr-calendar')
     expect(datetime_picker).to_have_count(1)
@@ -301,7 +298,7 @@ def test_datetimepicker_default(page, port, weekdays_as_str, months_as_str):
     assert valid_next_time(sec_before_click, sec_after_click, max_value=60, amount=5)
 
 
-def test_datetimepicker_value(page, port, march_2021, datetime_value_data):
+def test_datetimepicker_value(page, march_2021, datetime_value_data):
 
     year, month, day, hour, min, sec, month_str, date_str, datetime_str = datetime_value_data
 
@@ -310,9 +307,8 @@ def test_datetimepicker_value(page, port, march_2021, datetime_value_data):
     datetime_picker_widget = DatetimePicker(
         value=datetime.datetime(year, month, day, hour, min, sec)
     )
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     datetime_value = page.locator('.flatpickr-input')
     assert datetime_value.input_value() == datetime_str
@@ -372,15 +368,14 @@ def test_datetimepicker_value(page, port, march_2021, datetime_value_data):
     assert valid_prev_time(sec_before_click, sec_after_click, max_value=60, amount=5)
 
 
-def test_datetimepicker_start_end(page, port, march_2021, datetime_start_end):
+def test_datetimepicker_start_end(page, march_2021, datetime_start_end):
     start, end, selectable_dates = datetime_start_end
 
     march_2021_str, num_days, _, _ = march_2021
 
     datetime_picker_widget = DatetimePicker(start=start, end=end)
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     datetime_value = page.locator('.flatpickr-input')
     # click to show the datetime picker container
@@ -395,15 +390,14 @@ def test_datetimepicker_start_end(page, port, march_2021, datetime_start_end):
     expect(disabled_days).to_have_count(num_days - len(selectable_dates))
 
 
-def test_datetimepicker_disabled_dates(page, port, disabled_dates):
+def test_datetimepicker_disabled_dates(page, disabled_dates):
     active_date, disabled_list, disabled_str_list = disabled_dates
 
     datetime_picker_widget = DatetimePicker(
         disabled_dates=disabled_list, value=active_date
     )
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     # click to show the datetime picker container
     datetime_value = page.locator('.flatpickr-input')
@@ -416,14 +410,13 @@ def test_datetimepicker_disabled_dates(page, port, disabled_dates):
         assert disabled_days.nth(i).get_attribute('aria-label') == disabled_str_list[i]
 
 
-def test_datetimepicker_enabled_dates(page, port, march_2021, enabled_dates):
+def test_datetimepicker_enabled_dates(page, march_2021, enabled_dates):
     active_date, enabled_list, enabled_str_list = enabled_dates
     datetime_picker_widget = DatetimePicker(
         enabled_dates=enabled_list, value=active_date
     )
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     # click to show the datetime picker container
     datetime_value = page.locator('.flatpickr-input')
@@ -440,26 +433,23 @@ def test_datetimepicker_enabled_dates(page, port, march_2021, enabled_dates):
     expect(disabled_days).to_have_count(0)
 
 
-def test_datetimepicker_enable_time(page, port):
+def test_datetimepicker_enable_time(page):
     datetime_picker_widget = DatetimePicker(enable_time=False)
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     # click to show the datetime picker container
-    datetime_value = page.locator('.flatpickr-input')
-    datetime_value.dblclick()
+    page.locator('.flatpickr-input').dblclick()
 
     # no time editor
     time_editor = page.locator('.flatpickr-calendar .flatpickr-time.time24hr.hasSeconds')
     expect(time_editor).to_have_count(0)
 
 
-def test_datetimepicker_enable_seconds(page, port):
+def test_datetimepicker_enable_seconds(page):
     datetime_picker_widget = DatetimePicker(enable_seconds=False)
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     # click to show the datetime picker container
     datetime_value = page.locator('.flatpickr-input')
@@ -485,11 +475,10 @@ def test_datetimepicker_enable_seconds(page, port):
     expect(time_separators).to_have_count(1)
 
 
-def test_datetimepicker_military_time(page, port):
+def test_datetimepicker_military_time(page):
     datetime_picker_widget = DatetimePicker(military_time=False)
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     # click to show the datetime picker container
     datetime_value = page.locator('.flatpickr-input')
@@ -536,44 +525,40 @@ def test_datetimepicker_military_time(page, port):
     assert valid_next_time(hour_before_click, hour_after_click, max_value=13, amount=1)
 
 
-def test_datetimepicker_disable_editing(page, port):
+def test_datetimepicker_disable_editing(page):
     datetime_picker_widget = DatetimePicker(disabled=True)
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
+
     expect(page.locator('.flatpickr-input')).to_have_attribute('disabled', 'true')
 
 
-def test_datetimepicker_visible(page, port):
+def test_datetimepicker_visible(page):
     # add css class to search for name more easily
     datetime_picker_widget = DatetimePicker(
         visible=False, css_classes=['invisible-datetimepicker']
     )
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     expect(page.locator('.invisible-datetimepicker')).to_have_css('display', 'none')
 
 
-def test_datetimepicker_name(page, port):
+def test_datetimepicker_name(page):
     name = 'Datetime Picker'
     # add css class to search for name more easily
     datetime_picker_widget = DatetimePicker(
         name=name, css_classes=['datetimepicker-with-name']
     )
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+
+    serve_component(page, datetime_picker_widget)
 
     expect(page.locator('.datetimepicker-with-name > .bk-input-group > label')).to_have_text(name)
 
-def test_datetimepicker_no_value(page, port, datetime_start_end):
+def test_datetimepicker_no_value(page, datetime_start_end):
     datetime_picker_widget = DatetimePicker()
 
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+    serve_component(page, datetime_picker_widget)
 
     datetime_picker = page.locator('.flatpickr-input')
     assert datetime_picker.input_value() == ""
@@ -585,12 +570,10 @@ def test_datetimepicker_no_value(page, port, datetime_start_end):
     wait_until(lambda: datetime_picker.input_value() == '', page)
 
 
-def test_datetimerangepicker_no_value(page, port, datetime_start_end):
+def test_datetimerangepicker_no_value(page, datetime_start_end):
     datetime_picker_widget = DatetimeRangePicker()
 
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+    serve_component(page, datetime_picker_widget)
 
     datetime_picker = page.locator('.flatpickr-input')
     assert datetime_picker.input_value() == ""
@@ -600,15 +583,14 @@ def test_datetimerangepicker_no_value(page, port, datetime_start_end):
     wait_until(lambda: datetime_picker.input_value() == expected, page)
 
     datetime_picker_widget.value = None
+
     wait_until(lambda: datetime_picker.input_value() == '', page)
 
 
-def test_datetimepicker_remove_value(page, port, datetime_start_end):
+def test_datetimepicker_remove_value(page, datetime_start_end):
     datetime_picker_widget = DatetimePicker(value=datetime_start_end[0])
 
-    serve(datetime_picker_widget, port=port, threaded=True, show=False)
-    time.sleep(0.2)
-    page.goto(f"http://localhost:{port}")
+    serve_component(page, datetime_picker_widget)
 
     datetime_picker = page.locator('.flatpickr-input')
     assert datetime_picker.input_value() == "2021-03-02 00:00:00"
@@ -619,3 +601,71 @@ def test_datetimepicker_remove_value(page, port, datetime_start_end):
     datetime_picker.press("Escape")
 
     wait_until(lambda: datetime_picker_widget.value is None, page)
+
+
+def test_text_area_auto_grow_init(page):
+    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n")
+
+    serve_component(page, text_area)
+
+    expect(page.locator('.bk-input')).to_have_js_property('rows', 5)
+
+
+def test_text_area_auto_grow(page):
+    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n")
+
+    serve_component(page, text_area)
+
+    input_area = page.locator('.bk-input')
+    input_area.click()
+    input_area.press('Enter')
+    input_area.press('Enter')
+    input_area.press('Enter')
+
+    expect(page.locator('.bk-input')).to_have_js_property('rows', 8)
+
+
+def test_text_area_auto_grow_max_rows(page):
+    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n", max_rows=7)
+
+    serve_component(page, text_area)
+
+    input_area = page.locator('.bk-input')
+    input_area.click()
+    input_area.press('Enter')
+    input_area.press('Enter')
+    input_area.press('Enter')
+
+    expect(page.locator('.bk-input')).to_have_js_property('rows', 7)
+
+
+def test_text_area_auto_grow_min_rows(page):
+    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n", rows=3)
+
+    serve_component(page, text_area)
+
+    input_area = page.locator('.bk-input')
+    input_area.click()
+    for _ in range(5):
+        input_area.press('ArrowDown')
+    for _ in range(10):
+        input_area.press('Backspace')
+
+    expect(page.locator('.bk-input')).to_have_js_property('rows', 3)
+
+
+def test_text_area_auto_grow_shrink_back_on_new_value(page):
+    text_area = TextAreaInput(auto_grow=True, value="1\n2\n3\n4\n", max_rows=5)
+
+    serve_component(page, text_area)
+
+    input_area = page.locator('.bk-input')
+    input_area.click()
+    for _ in range(5):
+        input_area.press('ArrowDown')
+    for _ in range(10):
+        input_area.press('Backspace')
+
+    text_area.value = ""
+
+    expect(page.locator('.bk-input')).to_have_js_property('rows', 2)
