@@ -1,4 +1,5 @@
 from io import StringIO
+from pathlib import Path
 
 import pytest
 
@@ -78,10 +79,8 @@ def test_file_download_file():
 
 
 def test_file_download_callback():
-    file_download = FileDownload(callback=lambda: StringIO("data"), file="abc")
-
     with pytest.raises(ValueError):
-        file_download._clicks += 1
+        FileDownload(callback=lambda: StringIO("data"))
 
     file_download = FileDownload(callback=lambda: StringIO("data"), filename="abc.py")
 
@@ -131,3 +130,14 @@ def test_file_download_data():
     file_download.data = None
     file_download._clicks += 1
     assert file_download.data is not None
+
+def test_file_path_download():
+    path = Path(__file__)
+
+    file_download = FileDownload(path)
+
+    assert file_download.filename == "test_misc.py"
+    assert file_download.label == "Download test_misc.py"
+    file_download._clicks += 1
+    assert file_download.data
+    assert file_download._transfers == 1
