@@ -1,10 +1,8 @@
 import datetime as dt
-import time
 
 import numpy as np
 import pandas as pd
 import pytest
-import requests
 
 from bokeh.models.widgets.tables import (
     AvgAggregator, CellEditor, CheckboxEditor, DataCube, DateEditor,
@@ -17,9 +15,10 @@ from pandas._testing import (
 )
 
 from panel.depends import bind
-from panel.io.server import serve
 from panel.io.state import set_curdoc
 from panel.models.tabulator import CellClickEvent, TableEditEvent
+from panel.tests.util import serve_and_request, wait_until
+from panel.util import BOKEH_JS_NAT
 from panel.widgets import Button, TextInput
 from panel.widgets.tables import DataFrame, Tabulator
 
@@ -961,6 +960,86 @@ def test_tabulator_patch_with_dataframe(document, comm):
         if col != 'index':
             np.testing.assert_array_equal(table.value[col].values, expected[col])
 
+def test_tabulator_patch_with_dataframe_custom_index(document, comm):
+    df = pd.DataFrame(dict(A=[1, 4, 2]), index=['foo1', 'foo2', 'foo3'])
+    df_patch = pd.DataFrame(dict(A=[10]), index=['foo2'])
+
+    table = Tabulator(df)
+
+    model = table.get_root(document, comm)
+
+    table.patch(df_patch)
+
+    expected = {
+        'index': np.array(['foo1', 'foo2', 'foo3']),
+        'A': np.array([1, 10, 2]),
+    }
+    for col, values in model.source.data.items():
+        expected_array = expected[col]
+        np.testing.assert_array_equal(values, expected_array)
+        if col != 'index':
+            np.testing.assert_array_equal(table.value[col].values, expected[col])
+
+def test_tabulator_patch_with_dataframe_custom_index_name(document, comm):
+    df = pd.DataFrame(dict(A=[1, 4, 2]), index=['foo1', 'foo2', 'foo3'])
+    df.index.name = 'foo'
+    df_patch = pd.DataFrame(dict(A=[10]), index=['foo2'])
+    df.index.name = 'foo'
+
+    table = Tabulator(df)
+
+    model = table.get_root(document, comm)
+
+    table.patch(df_patch)
+
+    expected = {
+        'foo': np.array(['foo1', 'foo2', 'foo3']),
+        'A': np.array([1, 10, 2]),
+    }
+    for col, values in model.source.data.items():
+        expected_array = expected[col]
+        np.testing.assert_array_equal(values, expected_array)
+        if col != 'foo':
+            np.testing.assert_array_equal(table.value[col].values, expected[col])
+
+def test_tabulator_patch_with_complete_dataframe_custom_index(document, comm):
+    df = makeMixedDataFrame()[['A', 'B', 'C']]
+    df.index = [0, 1, 2, 3, 10]
+
+    table = Tabulator(df)
+
+    model = table.get_root(document, comm)
+
+    table.patch(df)
+
+    expected = {
+        'index': np.array([0, 1, 2, 3, 10]),
+        'A': np.array([0, 1, 2, 3, 4]),
+        'B': np.array([0, 1, 0, 1, 0]),
+        'C': np.array(['foo1', 'foo2', 'foo3', 'foo4', 'foo5']),
+    }
+    for col, values in model.source.data.items():
+        expected_array = expected[col]
+        np.testing.assert_array_equal(values, expected_array)
+        if col != 'index':
+            np.testing.assert_array_equal(table.value[col].values, expected[col])
+
+def test_tabulator_patch_with_dataframe_custom_index_multiple_error(document, comm):
+    df = pd.DataFrame(dict(A=[1, 4, 2]), index=['foo1', 'foo1', 'foo3'])
+    # Copy to assert at the end that the original dataframe hasn't been touched
+    original = df.copy()
+    df_patch = pd.DataFrame(dict(A=[20, 10]), index=['foo1', 'foo1'])
+
+    table = Tabulator(df)
+
+    with pytest.raises(
+        ValueError,
+        match=r"Patching a table with duplicate index values is not supported\. Found this duplicate index: 'foo1'"
+    ):
+        table.patch(df_patch)
+
+    pd.testing.assert_frame_equal(table.value, original)
+
 def test_tabulator_patch_with_dataframe_not_as_index(document, comm):
     df = makeMixedDataFrame().sort_values('A', ascending=False)
     table = Tabulator(df)
@@ -1193,6 +1272,51 @@ def test_tabulator_patch_ranges(document, comm):
         if col != 'index':
             np.testing.assert_array_equal(table.value[col].values, expected[col])
 
+def test_tabulator_patch_with_timestamp(document, comm):
+    # https://github.com/holoviz/panel/issues/5555
+    df = pd.DataFrame(dict(A=pd.to_datetime(['1980-01-01', '1980-01-02'])))
+    table = Tabulator(df)
+
+    model = table.get_root(document, comm)
+
+    table.patch({'A': [(0, pd.Timestamp('2021-01-01'))]})
+
+    expected = {
+        'index': np.array([0, 1]),
+        'A': np.array(['2021-01-01T00:00:00.000000000',
+                       '1980-01-02T00:00:00.000000000'],
+                      dtype='datetime64[ns]')
+    }
+    for col, values in model.source.data.items():
+        if col == 'A':
+            expected_array = expected[col].astype(np.int64) / 10e5
+        else:
+            expected_array = expected[col]
+        np.testing.assert_array_equal(values, expected_array)
+        if col != 'index':
+            np.testing.assert_array_equal(table.value[col].values, expected[col])
+
+def test_tabulator_patch_with_NaT(document, comm):
+    df = pd.DataFrame(dict(A=pd.to_datetime(['1980-01-01', np.nan])))
+    assert df.loc[1, 'A'] is pd.NaT
+    table = Tabulator(df)
+
+    model = table.get_root(document, comm)
+
+    table.patch({'A': [(0, pd.NaT)]})
+
+    # We're also checking that the NaT value that was in the original table
+    # at .loc[1, 'A'] is converted in the model as BOKEH_JS_NAT.
+    expected = {
+        'index': np.array([0, 1]),
+        'A': np.array([BOKEH_JS_NAT, BOKEH_JS_NAT])
+    }
+    for col, values in model.source.data.items():
+        expected_array = expected[col]
+        np.testing.assert_array_equal(values, expected_array)
+        # Not checking that the data in table.value is the same as expected
+        # In table.value we have NaT values, in expected the BOKEH_JS_NAT constant.
+
 
 def test_tabulator_stream_series_paginated_not_follow(document, comm):
     df = makeMixedDataFrame()
@@ -1799,17 +1923,13 @@ def test_tabulator_patch_event():
             table._process_event(event)
             assert values[-1] == (col, row, df[col].iloc[row])
 
-def test_server_edit_event(port):
+def test_server_edit_event():
     df = makeMixedDataFrame()
     table = Tabulator(df)
 
-    serve(table, port=port, threaded=True, show=False)
+    serve_and_request(table)
 
-    time.sleep(0.5)
-
-    requests.get(f'http://localhost:{port}')
-
-    assert table._models
+    wait_until(lambda: bool(table._models))
     ref, (model, _) = list(table._models.items())[0]
     doc = list(table._documents.keys())[0]
 
@@ -1822,8 +1942,7 @@ def test_server_edit_event(port):
     table._server_change(doc, ref, None, 'data', model.source.data, new_data)
     table._server_event(doc, TableEditEvent(model, 'B', 1))
 
-    time.sleep(0.1)
-    assert len(events) == 1
+    wait_until(lambda: len(events) == 1)
     assert events[0].value == 3.14
     assert events[0].old == 1
 
@@ -1841,7 +1960,7 @@ def test_tabulator_cell_click_event():
             table._process_event(event)
             assert values[-1] == (col, row, data[col].iloc[row])
 
-def test_server_cell_click_async_event(port):
+def test_server_cell_click_async_event():
     df = makeMixedDataFrame()
     table = Tabulator(df)
 
@@ -1855,26 +1974,20 @@ def test_server_cell_click_async_event(port):
 
     table.on_click(cb)
 
-    serve(table, port=port, threaded=True, show=False)
+    serve_and_request(table)
 
-    # Wait for server to start
-    time.sleep(1)
-
-    requests.get(f"http://localhost:{port}/")
+    wait_until(lambda: bool(table._models))
+    doc = list(table._models.values())[0][0].document
 
     data = df.reset_index()
-    doc = list(table._models.values())[0][0].document
     with set_curdoc(doc):
         for col in data.columns:
             for row in range(len(data)):
                 event = CellClickEvent(model=None, column=col, row=row)
                 table._process_event(event)
 
-    # Wait for callbacks to be scheduled
-    time.sleep(2)
-
     # Ensure multiple callbacks started concurrently
-    assert max(counts) > 1
+    wait_until(lambda: len(counts) >= 1 and max(counts) > 1)
 
 def test_tabulator_pagination_remote_cell_click_event():
     df = makeMixedDataFrame()
