@@ -275,6 +275,18 @@ def base64url_decode(input):
     return base64.urlsafe_b64decode(input)
 
 
+def decode_token(token, signed=True):
+    """
+    Decodes a signed or unsigned JWT token.
+    """
+    if signed and "." in token:
+        signing_input, _ = token.encode('utf-8').rsplit(b".", 1)
+        _, payload_segment = signing_input.split(b".", 1)
+    else:
+        payload_segment = token
+    return json.loads(base64url_decode(payload_segment).decode('utf-8'))
+
+
 class classproperty:
 
     def __init__(self, f):
@@ -317,17 +329,6 @@ def edit_readonly(parameterized: param.Parameterized) -> Iterator:
             p.constant = constant
 
 
-def eval_function(function):
-    args, kwargs = (), {}
-    if hasattr(function, '_dinfo'):
-        arg_deps = function._dinfo['dependencies']
-        kw_deps = function._dinfo.get('kw', {})
-        if kw_deps or any(isinstance(d, param.Parameter) for d in arg_deps):
-            args = (getattr(dep.owner, dep.name) for dep in arg_deps)
-            kwargs = {n: getattr(dep.owner, dep.name) for n, dep in kw_deps.items()}
-    return function(*args, **kwargs)
-
-
 def lazy_load(module, model, notebook=False, root=None, ext=None):
     from ..config import panel_extension as extension
     from ..io.state import state
@@ -474,3 +475,29 @@ def param_watchers(parameterized, value=_unset):
             parameterized.param.watchers = value
         else:
             return parameterized.param.watchers
+
+
+def flatten(line):
+    """
+    Flatten an arbitrarily nested sequence.
+
+    Inspired by: pd.core.common.flatten
+
+    Parameters
+    ----------
+    line : sequence
+        The sequence to flatten
+
+    Notes
+    -----
+    This only flattens list, tuple, and dict sequences.
+
+    Returns
+    -------
+    flattened : generator
+    """
+    for element in line:
+        if any(isinstance(element, tp) for tp in (list, tuple, dict)):
+            yield from flatten(element)
+        else:
+            yield element
