@@ -43,7 +43,7 @@ from .io.notebook import (
     render_mimebundle, render_model, show_server,
 )
 from .io.save import save
-from .io.state import curdoc_locked, state
+from .io.state import curdoc_locked, set_curdoc, state
 from .util import escape, param_reprs
 from .util.warnings import deprecated
 
@@ -79,7 +79,7 @@ class Layoutable(param.Parameterized):
     background = param.Parameter(default=None, doc="""
         Background color of the component.""")
 
-    css_classes = param.List(default=[], doc="""
+    css_classes = param.List(default=[], nested_refs=True, doc="""
         CSS classes to apply to the layout.""")
 
     design = param.ObjectSelector(default=None, objects=[], doc="""
@@ -106,15 +106,15 @@ class Layoutable(param.Parameterized):
         be specified as a two-tuple of the form (vertical, horizontal)
         or a four-tuple (top, right, bottom, left).""")
 
-    styles = param.Dict(default={}, doc="""
+    styles = param.Dict(default={}, nested_refs=True, doc="""
         Dictionary of CSS rules to apply to DOM node wrapping the
         component.""")
 
-    stylesheets = param.List(default=[], doc="""
+    stylesheets = param.List(default=[], nested_refs=True, doc="""
         List of stylesheets defined as URLs pointing to .css files
         or raw CSS defined as a string.""")
 
-    tags = param.List(default=[], doc="""
+    tags = param.List(default=[], nested_refs=True, doc="""
         List of arbitrary tags to add to the component.
         Can be useful for templating or for storing metadata on
         the model.""")
@@ -321,11 +321,12 @@ class ServableMixin:
         from .io.location import Location
         if isinstance(location, Location):
             loc = location
+            state._locations[doc] = loc
         elif doc in state._locations:
             loc = state._locations[doc]
         else:
-            loc = Location()
-        state._locations[doc] = loc
+            with set_curdoc(doc):
+                loc = state.location
         if root is None:
             loc_model = loc.get_root(doc)
         else:
@@ -725,7 +726,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         # Warning
         prev = f'{type(self).name}(..., background={self.background!r})'
         new = f"{type(self).name}(..., styles={{'background': {self.background!r}}})"
-        deprecated("1.3", prev, new)
+        deprecated("1.4", prev, new)
 
         self.styles = dict(self.styles, background=self.background)
 
@@ -861,7 +862,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         """
         Prints a compositional repr of the class.
         """
-        deprecated('1.3', f'{type(self).__name__}.pprint', 'print')
+        deprecated('1.4', f'{type(self).__name__}.pprint', 'print')
         print(self)
 
     def select(
@@ -899,7 +900,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         port: int (optional, default=0)
           Allows specifying a specific port
         """
-        deprecated('1.3', f'{type(self).__name__}.app', 'panel.io.notebook.show_server')
+        deprecated('1.4', f'{type(self).__name__}.app', 'panel.io.notebook.show_server')
         return show_server(self, notebook_url, port)
 
     def embed(
