@@ -9,13 +9,14 @@ import math
 
 from typing import (
     TYPE_CHECKING, Any, Callable, ClassVar, Dict, List, Mapping, Optional,
-    Tuple, Type,
+    Tuple, Type, TypeVar,
 )
 
 import param  # type: ignore
 
 from bokeh.models import ImportedStyleSheet, Tooltip
 from bokeh.models.dom import HTML
+from param.parameterized import register_reference_transform
 
 from .._param import Margin
 from ..layout.base import Row
@@ -23,14 +24,14 @@ from ..reactive import Reactive
 from ..viewable import Layoutable, Viewable
 
 if TYPE_CHECKING:
-
     from bokeh.document import Document
     from bokeh.model import Model
     from pyviz_comms import Comm
-    from typing_extensions import Self
 
     from ..layout.base import ListPanel
 
+    T = TypeVar('T', bound='Widget')
+
 
 class Widget(Reactive):
     """
@@ -74,7 +75,7 @@ class Widget(Reactive):
         super().__init__(**params)
 
     @classmethod
-    def from_param(cls, parameter: param.Parameter, **params) -> Self:
+    def from_param(cls, parameter: param.Parameter, **params) -> T:
         """
         Construct a widget from a Parameter and link the two
         bi-directionally.
@@ -104,6 +105,10 @@ class Widget(Reactive):
             props.remove('description')
         return tuple(props)
 
+    @property
+    def rx(self):
+        return self.param.value.rx
+
     def _process_param_change(self, params: Dict[str, Any]) -> Dict[str, Any]:
         params = super()._process_param_change(params)
         if self._widget_type is not None and 'stylesheets' in params:
@@ -244,3 +249,9 @@ class CompositeWidget(Widget):
     @property
     def _synced_params(self) -> List[str]:
         return []
+
+
+def _widget_transform(obj):
+    return obj.param.value if isinstance(obj, Widget) else obj
+
+register_reference_transform(_widget_transform)
