@@ -5,20 +5,20 @@ events or merely toggling between on-off states.
 from __future__ import annotations
 
 from typing import (
-    TYPE_CHECKING, Any, Callable, ClassVar, Dict, List, Mapping, Optional,
-    Type,
+    TYPE_CHECKING, Any, Awaitable, Callable, ClassVar, Dict, List, Mapping,
+    Optional, Type,
 )
 
 import param
 
 from bokeh.events import ButtonClick, MenuItemClick
-from bokeh.models import (
-    Button as _BkButton, Dropdown as _BkDropdown, Toggle as _BkToggle,
-)
+from bokeh.models import Dropdown as _BkDropdown, Toggle as _BkToggle
 from bokeh.models.ui import SVGIcon, TablerIcon
 
 from ..io.resources import CDN_DIST
 from ..links import Callback
+from ..models.widgets import Button as _BkButton
+from ._mixin import TooltipMixin
 from .base import Widget
 
 if TYPE_CHECKING:
@@ -156,7 +156,7 @@ class _ClickButton(_ButtonBase, IconMixin):
         return Callback(self, code=callbacks, args=args)
 
 
-class Button(_ClickButton):
+class Button(_ClickButton, TooltipMixin):
     """
     The `Button` widget allows triggering events when the button is
     clicked.
@@ -181,11 +181,15 @@ class Button(_ClickButton):
         Toggles from False to True while the event is being processed.""")
 
     _rename: ClassVar[Mapping[str, str | None]] = {
-        'clicks': None, 'name': 'label', 'value': None
+        'clicks': None, 'name': 'label', 'value': None,
+    }
+
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'button_style': None, 'description': None
     }
 
     _target_transforms: ClassVar[Mapping[str, str | None]] = {
-        'event:button_click': None, 'value': None
+        'event:button_click': None, 'value': None,
     }
 
     _widget_type: ClassVar[Type[Model]] = _BkButton
@@ -243,17 +247,26 @@ class Button(_ClickButton):
         self.clicks += 1
 
     def on_click(
-        self, callback: Callable[[param.parameterized.Event], None]
+        self, callback: Callable[[param.parameterized.Event], None | Awaitable[None]]
     ) -> param.parameterized.Watcher:
         """
         Register a callback to be executed when the `Button` is clicked.
 
         The callback is given an `Event` argument declaring the number of clicks
 
+        Example
+        -------
+
+        >>> button = pn.widgets.Button(name='Click me')
+        >>> def handle_click(event):
+        ...    print("I was clicked!")
+        >>> button.on_click(handle_click)
+
         Arguments
         ---------
-        callback: (Callable[[param.parameterized.Event], None])
-            The function to run on click events. Must accept a positional `Event` argument
+        callback:
+            The function to run on click events. Must accept a positional `Event` argument. Can
+            be a sync or async function
 
         Returns
         -------
