@@ -14,7 +14,7 @@ import numpy as np
 import param
 
 from bokeh.model import Model
-from bokeh.models import ColumnDataSource, ImportedStyleSheet, Selection
+from bokeh.models import ColumnDataSource, ImportedStyleSheet
 from bokeh.models.widgets.tables import (
     AvgAggregator, CellEditor, CellFormatter, CheckboxEditor, DataCube,
     DataTable, DateEditor, DateFormatter, GroupingInfo, IntEditor,
@@ -24,12 +24,13 @@ from bokeh.models.widgets.tables import (
 from bokeh.util.serialization import convert_datetime_array
 from pyviz_comms import JupyterComm
 
-from ..depends import param_value_if_widget
+from ..depends import transform_reference
 from ..io.resources import CDN_DIST, CSS_URLS
 from ..io.state import state
 from ..reactive import Reactive, ReactiveData
 from ..util import (
-    clone_model, isdatetime, lazy_load, updating,
+    BOKEH_JS_NAT, clone_model, datetime_as_utctimestamp, isdatetime, lazy_load,
+    updating,
 )
 from .base import Widget
 from .button import Button
@@ -52,18 +53,18 @@ def _convert_datetime_array_ignore_list(v):
 
 class BaseTable(ReactiveData, Widget):
 
-    aggregators = param.Dict(default={}, doc="""
+    aggregators = param.Dict(default={}, nested_refs=True, doc="""
         A dictionary mapping from index name to an aggregator to
         be used for hierarchical multi-indexes (valid aggregators
         include 'min', 'max', 'mean' and 'sum'). If separate
         aggregators for different columns are required the dictionary
         may be nested as `{index_name: {column_name: aggregator}}`""")
 
-    editors = param.Dict(default={}, doc="""
+    editors = param.Dict(default={}, nested_refs=True, doc="""
         Bokeh CellEditor to use for a particular column
         (overrides the default chosen based on the type).""")
 
-    formatters = param.Dict(default={}, doc="""
+    formatters = param.Dict(default={}, nested_refs=True, doc="""
         Bokeh CellFormatter to use for a particular column
         (overrides the default chosen based on the type).""")
 
@@ -82,14 +83,14 @@ class BaseTable(ReactiveData, Widget):
     sorters = param.List(default=[], doc="""
         A list of sorters to apply during pagination.""")
 
-    text_align = param.ClassSelector(default={}, class_=(dict, str), doc="""
+    text_align = param.ClassSelector(default={}, nested_refs=True, class_=(dict, str), doc="""
         A mapping from column name to alignment or a fixed column
         alignment, which should be one of 'left', 'center', 'right'.""")
 
-    titles = param.Dict(default={}, doc="""
+    titles = param.Dict(default={}, nested_refs=True, doc="""
         A mapping from column name to a title to override the name with.""")
 
-    widths = param.ClassSelector(default={}, class_=(dict, int), doc="""
+    widths = param.ClassSelector(default={}, nested_refs=True, class_=(dict, int), doc="""
         A mapping from column name to column width or a fixed column
         width.""")
 
@@ -302,9 +303,8 @@ class BaseTable(ReactiveData, Widget):
     def _get_properties(self, doc: Document) -> Dict[str, Any]:
         properties = super()._get_properties(doc)
         properties['columns'] = self._get_columns()
-        properties['source']  = ColumnDataSource(
-            data=self._data, selected=Selection(indices=self.selection)
-        )
+        properties['source']  = cds = ColumnDataSource(data=self._data)
+        cds.selected.indices = self.selection
         return properties
 
     def _get_model(
@@ -544,7 +544,7 @@ class BaseTable(ReactiveData, Widget):
         elif isinstance(filter, (FunctionType, MethodType)):
             deps = list(filter._dinfo['kw'].values()) if hasattr(filter, '_dinfo') else []
         else:
-            filter = param_value_if_widget(filter)
+            filter = transform_reference(filter)
             if not isinstance(filter, param.Parameter):
                 raise ValueError(f'{type(self).__name__} filter must be '
                                  'a constant value, parameter, widget '
@@ -805,15 +805,31 @@ class BaseTable(ReactiveData, Widget):
             self.patch(patch_value_dict, as_index=as_index)
         elif isinstance(patch_value, dict):
             columns = list(self.value.columns)
+            patches = {}
             for k, v in patch_value.items():
-                for (ind, value) in v:
-                    if isinstance(ind, slice):
-                        ind = range(ind.start, ind.stop, ind.step or 1)
+                values = []
+                for (patch_ind, value) in v:
+                    data_ind = patch_ind
+                    if isinstance(patch_ind, slice):
+                        data_ind = range(patch_ind.start, patch_ind.stop, patch_ind.step or 1)
                     if as_index:
-                        self.value.loc[ind, k] = value
+                        if not isinstance(data_ind, range):
+                            patch_ind = self.value.index.get_loc(patch_ind)
+                            if not isinstance(patch_ind, int):
+                                raise ValueError(
+                                    'Patching a table with duplicate index values is not supported. '
+                                    f'Found this duplicate index: {data_ind!r}'
+                                )
+                        self.value.loc[data_ind, k] = value
                     else:
-                        self.value.iloc[ind, columns.index(k)] = value
-            self._patch(patch_value)
+                        self.value.iloc[data_ind, columns.index(k)] = value
+                    if isinstance(value, pd.Timestamp):
+                        value = datetime_as_utctimestamp(value)
+                    elif value is pd.NaT:
+                        value = BOKEH_JS_NAT
+                    values.append((patch_ind, value))
+                patches[k] = values
+            self._patch(patches)
         else:
             raise ValueError(
                 f"Patching with a patch_value of type {type(patch_value).__name__} "
@@ -997,11 +1013,11 @@ class Tabulator(BaseTable):
     >>> Tabulator(df, theme='site', pagination='remote', page_size=25)
     """
 
-    buttons = param.Dict(default={}, doc="""
+    buttons = param.Dict(default={}, nested_refs=True, doc="""
         Dictionary mapping from column name to a HTML element
         to use as the button icon.""")
 
-    expanded = param.List(default=[], doc="""
+    expanded = param.List(default=[], nested_refs=True, doc="""
         List of expanded rows, only applicable if a row_content function
         has been defined.""")
 
@@ -1013,31 +1029,31 @@ class Tabulator(BaseTable):
         List of client-side filters declared as dictionaries containing
         'field', 'type' and 'value' keys.""")
 
-    frozen_columns = param.List(default=[], doc="""
+    frozen_columns = param.List(default=[], nested_refs=True, doc="""
         List indicating the columns to freeze. The column(s) may be
         selected by name or index.""")
 
-    frozen_rows = param.List(default=[], doc="""
+    frozen_rows = param.List(default=[], nested_refs=True, doc="""
         List indicating the rows to freeze. If set, the
         first N rows will be frozen, which prevents them from scrolling
         out of frame; if set to a negative value the last N rows will be
         frozen.""")
 
-    groups = param.Dict(default={}, doc="""
+    groups = param.Dict(default={}, nested_refs=True, doc="""
         Dictionary mapping defining the groups.""")
 
-    groupby = param.List(default=[], doc="""
+    groupby = param.List(default=[], nested_refs=True, doc="""
         Groups rows in the table by one or more columns.""")
 
-    header_align = param.ClassSelector(default={}, class_=(dict, str), doc="""
+    header_align = param.ClassSelector(default={}, nested_refs=True, class_=(dict, str), doc="""
         A mapping from column name to alignment or a fixed column
         alignment, which should be one of 'left', 'center', 'right'.""")
 
-    header_filters = param.ClassSelector(class_=(bool, dict), doc="""
+    header_filters = param.ClassSelector(class_=(bool, dict), nested_refs=True, doc="""
         Whether to enable filters in the header or dictionary
         configuring filters for each column.""")
 
-    hidden_columns = param.List(default=[], doc="""
+    hidden_columns = param.List(default=[], nested_refs=True, doc="""
         List of columns to hide.""")
 
     layout = param.ObjectSelector(default='fit_data_table', objects=[
@@ -1091,11 +1107,11 @@ class Tabulator(BaseTable):
         ], doc="""
         Tabulator CSS theme to apply to table.""")
 
-    theme_classes = param.List(default=[], item_type=str, doc="""
+    theme_classes = param.List(default=[], nested_refs=True, item_type=str, doc="""
        List of extra CSS classes to apply to the Tabulator element
        to customize the theme.""")
 
-    title_formatters = param.Dict(default={}, doc="""
+    title_formatters = param.Dict(default={}, nested_refs=True, doc="""
        Tabulator formatter specification to use for a particular column
        header title.""")
 
